unit FormMainUnit;
{ This code was written by
    Ulf Grasse, SEPT 16, 1996.
  Please consider this code public domain (i.e. Freeware), but abide
  by the following rules:
  - If you find this code useful please email me or (better!) send me a
    postcard from your hometown.
  - If you have questions regarding this code and/or techniques and
    methods used in it, please feel free to email me with same.  I can
    NOT guarantee a prompt answer, but I will try my best to keep up.
    Questions and inquiries which are accompanied or preceeded by a
    postcard receive priority handling. <smile>
    For replies I will *exclusively* use email.  Be sure to include your
    e-mail address should you send a request via snail-mail or fax.
  - If you decide to add more features and make this a "real" program,
    I'd be interested to see the results.  Please email me a .zip!
  - If you this code or any part of same in a shareware or commercial
    program, you are required to provide me with a roundtrip-ticket
    and one week hotel reservation as soon as you make your first MILLION
    on it... as if that'll ever happen.
  - You are required to smile while using this code or any part of it.

  You may contact me at:
    Ulf Grasse             e-mail: software@ulf.net
    705 King St.              URL: http://www.ulf.net
    Rockford, IL 61103
    USA                       Fax: 815-969-9476
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, IniFiles, ExtCtrls;

type
  TFormMain = class(TForm)
    Memo: TMemo;
    LeftPanel: TPanel;
    BtnScan: TButton;
    BtnGenerate: TButton;
    procedure BtnScanClick(Sender: TObject);
    procedure BtnGenerateClick(Sender: TObject);
  private
    { Private declarations }
    html: text;           { The output-file for generating HTML }
    writeHTML : boolean;  { are we generating HTML or just scanning? }
  public
    { Public declarations }
    { This procedure is recursively called for every directory }
    procedure ScanDir(Dir, Path : string; Indent : integer);
    { The following procedures generate the HTML file }
    procedure OpenHTML;
    procedure CloseHTML;
    procedure AddHeader(Name : string);
    procedure AddURL(Name, URL : string);
  end;

var
  FormMain: TFormMain;

implementation

{$R *.DFM}

const REC_FOUND = 0;                        { OK-Result of FindFirst  }
      ROOT_DIR  = 'C:\Windows\Favorites';   { Where to start looking  }
      DIR_EXT   = '\*.*';                   { mask for Directories    }
      URL_EXT   = '\*.URL';                 { mask for URLs           }

{ The .URL files used by IE have the same format that was used in     }
{ Windows 3.x's .INI files.  Thus, we can use the TIniFile class and  }
{ TIniFile.Readstring method to extract the information.              }
{ Methinks it should be possible to store additional information in   }
{ the .URL files for managing shortcuts, etc.  Play with it if you    }
{ have time.  Maybe set it to *not* export some shortcuts, or to      }
{ include pictures (IMG=), or set different fonts for HTML generation }

      INI_SECTION = 'InternetShortcut';     { Section in the URL file }
      INI_ITEM    = 'URL';                  { Item in the section     }
      INI_DEFAULT = 'N/A';                  { ...if URL not found     }

      STARTINDENT = 0;                      { Left Margin in Memo     }
      ADD_INDENT  = 4;                      { Indent for each level   }
      DIR_INDENT  = 2;                      { back up two for dir.    }

      MISCELLANEOUS = 'Miscellaneous';      { Header for Root Dir     }
      OUTPUTFILE  = 'C:\MyFavorites.HTM';   { Name of the html file   }


procedure TFormMain.ScanDir(Dir,            { Directory name for Hdr  }
                            Path : string;  { Path to search          }
                            Indent : integer); { Left Margin          }
var SearchRec : TSearchRec;
          Err : integer;
          URL : string;
begin { of ScanDir }
  { Scan for Directories first }
  Err:=FindFirst(Path+DIR_EXT, faDirectory, SearchRec);
  while (Err = REC_FOUND) do with SearchRec do begin
    if Name[1] <> '.' then begin { Avoid Directories '.' and '..' }
      ScanDir(Name, Path+'\'+Name, Indent+ADD_INDENT)  { Recurse down }
    end;
    Err:=FindNext(SearchRec)     { Find Next Directory name }
  end;
  FindClose(SearchRec);          { We're done here }

  { Scan for URLs }
  Err:=FindFirst(Path+URL_EXT, faAnyFile, SearchRec);
  { if a URL was found in this directory, output the directory name or
    generate and HTML Header with the directory name.  This assures that
    a the header will not be generated for empty directories, or non-URL
    directories. }
  if (Err = REC_FOUND) then begin
    if WriteHTML then begin
      if Dir>'' then AddHeader(Dir)          { Generate HTML }
                else AddHeader(MISCELLANEOUS)
    end
    else
      if Dir>'' then                             { Or Output in memo }
        Memo.Lines.Add(Format('%*s *%s',[Indent-DIR_INDENT, '', Dir]))
  end;
    { now begin scanning the dir }
  while (Err = REC_FOUND) do with SearchRec do begin
    { Get the URL information from the current URL file }
    with TIniFile.Create(Path+'\'+Name) do begin
      URL:=ReadString(INI_SECTION, INI_ITEM, INI_DEFAULT);
      Free
    end;
    { output or generate html for the name and URL }
    if WriteHTML then
      AddURL(Copy(Name, 1,Length(Name)-4), URL)  { Copy deletes the extension ".URL" }
    else begin
      Memo.Lines.Add(Format('%*s %s',[Indent, '', Copy(Name, 1,Length(Name)-4)]));
      Memo.Lines.Add(Format('%*s =  %s',[Indent, '', URL]))
    end;
    Err:=FindNext(SearchRec)
  end;
  FindClose(SearchRec)
  { OK, all done }
end;


procedure TFormMain.OpenHTML;
begin
  { Open the HTML file and write a standard HTML Header }
  { Feel free to customize this to match your page design }
  AssignFile(html, OUTPUTFILE);
  Rewrite(html);
  writeln(html,'<html><head><title>My Favorites</title>');
  writeln(html,'<base target="frame"></head>'); { might want to remove the 'target' }
  writeHTML:=true
end;

procedure TFormMain.AddHeader(Name : string);
begin
  { Write a header (i.e. Directory name ) }
  writeln(html,'<p><font size=5><strong>'+Name+'</strong></font></p>');
end;

procedure TFormMain.AddUrl(Name, URL : string);
begin
  { Write a URL }
  writeln(html,Format('<p><a href="%s"><strong>%s</strong></a></p>',[URL,Name]));
end;

procedure TFormMain.CloseHTML;
begin
  { Generate the usual HTML page ending, then close the file }
  writeln(html,'</body></html>');
  CloseFile(html);
  writeHTML:=false
end;


procedure TFormMain.BtnScanClick(Sender: TObject);
begin
  WriteHTML:=False;
  ScanDir('',ROOT_DIR,0)
end;


procedure TFormMain.BtnGenerateClick(Sender: TObject);
begin
  OpenHTML;
  ScanDir('','C:\Windows\Favorites',0);
  CloseHTML
end;

end.
